Class: Foxtail::Syntax::Parser::Indent

Inherits:
Struct
  • Object
show all
Defined in:
lib/foxtail/syntax/parser.rb

Overview

Define Indent as a Struct for temporary indentation tokens Note: Uses Struct instead of Data.define because the value field is mutated in dedent()

Instance Attribute Summary collapse

Instance Attribute Details

#endObject

Returns the value of attribute end

Returns:

  • (Object)

    the current value of end



13
14
15
# File 'lib/foxtail/syntax/parser.rb', line 13

def end
  @end
end

#spanObject

Returns the value of attribute span

Returns:

  • (Object)

    the current value of span



13
14
15
# File 'lib/foxtail/syntax/parser.rb', line 13

def span
  @span
end

#startObject

Returns the value of attribute start

Returns:

  • (Object)

    the current value of start



13
14
15
# File 'lib/foxtail/syntax/parser.rb', line 13

def start
  @start
end

#valueObject

Returns the value of attribute value

Returns:

  • (Object)

    the current value of value



13
14
15
# File 'lib/foxtail/syntax/parser.rb', line 13

def value
  @value
end