Class: Foxtail::Syntax::Parser::Indent
- Inherits:
-
Struct
- Object
- Struct
- Foxtail::Syntax::Parser::Indent
- Defined in:
- lib/foxtail/syntax/parser.rb
Overview
Define Indent as a Struct for temporary indentation tokens Note: Uses Struct instead of Data.define because the value field is mutated in dedent()
Instance Attribute Summary collapse
-
#end ⇒ Object
Returns the value of attribute end.
-
#span ⇒ Object
Returns the value of attribute span.
-
#start ⇒ Object
Returns the value of attribute start.
-
#value ⇒ Object
Returns the value of attribute value.
Instance Attribute Details
#end ⇒ Object
Returns the value of attribute end
13 14 15 |
# File 'lib/foxtail/syntax/parser.rb', line 13 def end @end end |
#span ⇒ Object
Returns the value of attribute span
13 14 15 |
# File 'lib/foxtail/syntax/parser.rb', line 13 def span @span end |
#start ⇒ Object
Returns the value of attribute start
13 14 15 |
# File 'lib/foxtail/syntax/parser.rb', line 13 def start @start end |
#value ⇒ Object
Returns the value of attribute value
13 14 15 |
# File 'lib/foxtail/syntax/parser.rb', line 13 def value @value end |